﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Globalization;

[assembly:CLSCompliant(true)]
namespace MP3Player
{
    public partial class PlayerForm : Form
    {
        private bool _repeat;
        private int _playlistIndex = -1;
        private PlayBack AudioVideo;
        private List<Object> PlayerControls = new List<Object>();
        private PlayList playlist = new PlayList();

        public PlayerForm()
        {
            InitializeComponent();
            AudioVideo = new PlayBack(videoRender);
            FindDisabledControls(this);
            PlayListBox.DataSource = playlist.InnerList;
        }

        private void FindDisabledControls(Control me)
        {
            foreach (Control c in me.Controls)
            {
                if (!c.Enabled)
                {
                    PlayerControls.Add(c);
                }
                else if (c is MenuStrip)
                {
                    foreach (ToolStripMenuItem i in ((MenuStrip)c).Items)
                    {
                        if (!i.Enabled)
                        {
                            PlayerControls.Add(i);
                        }
                        else
                        {
                            foreach (ToolStripItem f in i.DropDownItems)
                            {
                                if (!f.Enabled)
                                {
                                    PlayerControls.Add(f);
                                }
                            }
                        }
                    }
                }
                else if (c is ToolStrip)
                {
                    foreach (ToolStripItem i in ((ToolStrip)c).Items)
                    {
                        if (!i.Enabled)
                        {
                            PlayerControls.Add(i);
                        }
                    }
                }
                else
                {
                    FindDisabledControls(c);
                }
            }
        }

        private void setVolume()
        {
            //Nonlinear range from -10000 to 0
            volumeControl1.Enabled = true;
            if (tsbMute.Checked)
            {
                AudioVideo.Volume = -10000;
            }
            else
            {
                AudioVideo.Volume = -10000 + (int)(Math.Sqrt(Math.Sqrt((double)volumeControl1.Volume / 100d)) * 10000);
            }

        }

        private void volumeControl1_VolumeChanged(object sender, EventArgs e)
        {
            setVolume();
        }

        private void tsbMute_CheckStateChanged(object sender, EventArgs e)
        {
            setVolume();
        }

        private void startPlay()
        {
            tsbPlay.Checked = true;
            tsbPause.Checked = false;
            timSeekbarRefesh.Enabled = true;
            AudioVideo.Play();
            PlayListBox.Invalidate();
            refreshView();
        }

        private void stopPlay()
        {
            tsbPlay.Checked = false;
            tsbPause.Checked = false;
            timSeekbarRefesh.Enabled = false;
            AudioVideo.Stop();
            PlayListBox.Invalidate();
            refreshView();
        }

        private void togglePause()
        {
            if (tsbPlay.Checked || tsbPause.Checked)
            {
                tsbPlay.Checked = tsbPause.Checked;
                tsbPause.Checked = !tsbPause.Checked;
                if (tsbPause.Checked)
                    AudioVideo.Pause();
                else
                    AudioVideo.Play();
            }
            PlayListBox.Invalidate();
            refreshView();
        }

        private void closeFile()
        {
            SeekBar.Enabled = timSeekbarRefesh.Enabled = false;
            SeekBar.Value = 0;
            stopPlay();
            AudioVideo.FileName = "";
            enableControls(false);
        }

        private void tsbPlay_Click(object sender, EventArgs e)
        {
            startPlay();
        }

        private void tsbPause_Click(object sender, EventArgs e)
        {
            togglePause();
        }

        private void tsbStop_Click(object sender, EventArgs e)
        {
            stopPlay();
        }

        private void timSeekbarRefesh_Tick(object sender, EventArgs e)
        {
            refreshView();
            if (AudioVideo.CurrentPosition >= AudioVideo.Duration)
                playlistLoadNext();
        }

        private void refreshView()
        {
                SeekBar.Max = (AudioVideo.Duration);
                SeekBar.Value = (AudioVideo.CurrentPosition);
                tsslTime.Text = formatTime(AudioVideo.CurrentPosition, AudioVideo.Duration);
                tsslStatus.Text = (AudioVideo.State == PlayState.Running) ? "Abspielen" : "Angehalten";
        }

        private String formatTime(long current, long total)
        {
            DateTime currentTime = new DateTime(current);
            DateTime totalTime = new DateTime(total);
            IFormatProvider provider = DateTimeFormatInfo.CurrentInfo;
            if (totalTime.Hour == 0)
                return currentTime.ToString("mm:ss", provider) + " / " + totalTime.ToString("mm:ss", provider);
            return currentTime.ToString("HH:mm:ss", provider) + " / " + totalTime.ToString("HH:mm:ss", provider);
        }

        private void mnuFileOpen_Click(object sender, EventArgs e)
        {
            playlist.Clear();
            closeFile();
            openFilesAndAddToPlaylist(true);
        }

        private void openFilesAndAddToPlaylist(bool playNewFile)
        {
            OpenFileDialog ofd = new OpenFileDialog();
            ofd.InitialDirectory = Environment.GetFolderPath(Environment.SpecialFolder.MyMusic);
            ofd.Filter = @"Mediendateien|*.ogg;*.mp3;*.wav;*.wma;*.wmv;*.acc;*.mpg;*.avi;*.ac3;*.mpeg;*.divx;*.mkv;|Alle dateien|*.*";
            ofd.Multiselect = true;
            if (ofd.ShowDialog() == DialogResult.OK)
            {
                int newIndex = PlayListBox.Items.Count;
                foreach (String f in ofd.FileNames)
                {
                    playlist.AddSong(f);
                }
                if (playNewFile)
                    playlistLoadIndex(newIndex);
            }
        }


        private void mnuFileClose_Click(object sender, EventArgs e)
        {
            closeFile();
        }

        private void mnuStartPause_Click(object sender, EventArgs e)
        {
            togglePause();
        }

        private void mnuStop_Click(object sender, EventArgs e)
        {
            stopPlay();
        }

        private void mnuRepeat_Click(object sender, EventArgs e)
        {
            mnuRepeat.Checked = !mnuRepeat.Checked;
            _repeat = mnuRepeat.Checked;
        }

        private void tsbBackward_Click(object sender, EventArgs e)
        {
            if (AudioVideo != null && AudioVideo.CurrentPosition < 5)
                playlistLoadPrev();
            else
            {
                stopPlay();
                startPlay();
            }
        }

        private void tsbForward_Click(object sender, EventArgs e)
        {
            playlistLoadNext();
        }

        private void PlayListBox_DrawItem(object sender, DrawItemEventArgs e)
        {
            // Draw the background of the ListBox control for each item.
            e.DrawBackground();
            // Define the default color of the brush as black.
            Brush myBrush = Brushes.Black;

            // Determine the color of the brush to draw each item based 
            // on the index of the item to draw.
            if ((e.State & DrawItemState.Selected) != DrawItemState.Selected)
            {
                if (e.Index == _playlistIndex)
                    myBrush = new SolidBrush(Color.FromKnownColor(KnownColor.Highlight));
                else
                    myBrush = new SolidBrush(Color.FromKnownColor(KnownColor.ControlText));
            }
            else
            {
                myBrush = new SolidBrush(Color.FromKnownColor(KnownColor.HighlightText));
            }

            StringFormat s = new StringFormat(StringFormatFlags.NoWrap | StringFormatFlags.FitBlackBox | StringFormatFlags.NoClip);
            // Draw the current item text based on the current Font 
            // and the custom brush settings.
            Rectangle textRect = e.Bounds;
            textRect.X += 16;
            if (e.Index != -1)
            {
                String str = PlayListBox.Items[e.Index].ToString();

                e.Graphics.DrawString(str,
                    e.Font, myBrush, textRect, s);
                e.Graphics.SmoothingMode = System.Drawing.Drawing2D.SmoothingMode.AntiAlias;
                if (e.Index == _playlistIndex)
                {
                    Rectangle arrowRect = e.Bounds;
                    arrowRect.Width = 12;
                    if (tsbPlay.Checked == true)
                    {
                        //draw arrow;
                        PointF[] points = new PointF[3];
                        points[0].X = arrowRect.X + 3.5f;
                        points[0].Y = arrowRect.Y + 2;
                        points[1].X = arrowRect.X + 3.5f;
                        points[1].Y = arrowRect.Bottom - 2;
                        points[2].X = arrowRect.Right;
                        points[2].Y = (arrowRect.Bottom + arrowRect.Y) / 2;
                        e.Graphics.FillPolygon(myBrush, points);
                    }
                    else if (tsbPause.Checked == true)
                    {
                        //draw 2 rects (pause symbol);
                        PointF[] points = new PointF[4];
                        points[0].X = arrowRect.X + 3.5f;
                        points[0].Y = arrowRect.Y + 2.5f;
                        points[1].X = arrowRect.X + 3.5f;
                        points[1].Y = arrowRect.Bottom - 2.5f;
                        points[2].X = arrowRect.X + 6.5f;
                        points[2].Y = arrowRect.Bottom - 2.5f;
                        points[3].X = arrowRect.X + 6.5f;
                        points[3].Y = arrowRect.Y + 2.5f;
                        e.Graphics.FillPolygon(myBrush, points);

                        points[0].X = arrowRect.Right - 3.5f;
                        points[0].Y = arrowRect.Y + 2.5f;
                        points[1].X = arrowRect.Right - 3.5f;
                        points[1].Y = arrowRect.Bottom - 2.5f;
                        points[2].X = arrowRect.Right - 0.5f;
                        points[2].Y = arrowRect.Bottom - 2.5f;
                        points[3].X = arrowRect.Right - 0.5f;
                        points[3].Y = arrowRect.Y + 2.5f;
                        e.Graphics.FillPolygon(myBrush, points);
                    }
                    else
                    {
                        //draw pause symbol
                        PointF[] points = new PointF[4];
                        points[0].X = arrowRect.X + 3.5f;
                        points[0].Y = arrowRect.Y + 2.5f;
                        points[1].X = arrowRect.X + 3.5f;
                        points[1].Y = arrowRect.Bottom - 2.5f;
                        points[2].X = arrowRect.Right - 0.5f;
                        points[2].Y = arrowRect.Bottom - 2.5f;
                        points[3].X = arrowRect.Right - 0.5f;
                        points[3].Y = arrowRect.Y + 2.5f;
                        e.Graphics.FillPolygon(myBrush, points);
                    }
                }

                // If the ListBox has focus, draw a focus rectangle around the selected item.
                e.DrawFocusRectangle();
            }
        }

        private void mnuShowPlaylist_Click(object sender, EventArgs e)
        {
            PlayListPanel.Visible = !PlayListPanel.Visible;
            mnuShowPlaylist.Checked = PlayListPanel.Visible;
        }

        private void PlayListBox_SelectedIndexChanged(object sender, EventArgs e)
        {
            PlayListBox.Invalidate();
        }

        private void PlayListBox_MouseDoubleClick(object sender, MouseEventArgs e)
        {
            playlistLoadIndex(PlayListBox.SelectedIndex);
        }

        private void playlistLoadNext()
        {
            _playlistIndex++;
            if (_playlistIndex >= PlayListBox.Items.Count)
                _playlistIndex = 0;
            PlayListBox.Invalidate();
            if (_repeat || _playlistIndex != 0)
            {
                tryPlayFile(playlist.Item(_playlistIndex).Filename);
            }
            else
            {
                stopPlay();
            }
        }

        private void playlistLoadPrev()
        {
            _playlistIndex--;
            if (_playlistIndex < 0 && _repeat)
                _playlistIndex = PlayListBox.Items.Count - 1;
            PlayListBox.Invalidate();
            if (_playlistIndex >= 0)
            {
                tryPlayFile(playlist.Item(_playlistIndex).Filename);
            }
            else
            {
                _playlistIndex = 0;
                stopPlay();
            }
        }

        private void playlistLoadIndex(int index)
        {
            if (_playlistIndex == index && AudioVideo != null && AudioVideo.State == PlayState.Running)
                return;
            _playlistIndex = index;
            if (_playlistIndex >= PlayListBox.Items.Count)
                _playlistIndex = 0;
            PlayListBox.Invalidate();
            closeFile();
            if (_playlistIndex >= 0)
            {
                tryPlayFile(playlist.Item(_playlistIndex).Filename);
            }
        }

        private void tryPlayFile(String fileToPlay)
        {
            try
            {
                AudioVideo.FileName = fileToPlay;
            }
            catch (Microsoft.DirectX.DirectXException e)
            {
                MessageBox.Show("Konnte Datei nicht laden. Fehlercode: " + e.ErrorString,"Fehler",MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            if (AudioVideo.State != PlayState.NoData)
            {
                startPlay();
                enableControls(true);
            }
            else
            {
                enableControls(false);
            }
        }

        private void enableControls(bool value)
        {
            foreach (Object c in PlayerControls)
            {
                c.GetType().GetProperty("Enabled").SetValue(c,value,null);
            }
        }

        private void PlayListBox_Leave(object sender, EventArgs e)
        {
            PlayListBox.SelectedIndices.Clear();
        }

        private void abspielenToolStripMenuItem_Click(object sender, EventArgs e)
        {
            playlistLoadIndex(PlayListBox.SelectedIndex);
        }

        private void mnuPlaylistRemove_Click(object sender, EventArgs e)
        {
            List<int> toRemove = new List<int>();
            for (int i = 0; i < PlayListBox.SelectedItems.Count ; i++)
            {
                int index = PlayListBox.SelectedIndices[i];
                toRemove.Add(index);
            }
            foreach (int index in toRemove)
            {
                playlist.RemoveSong(index);
                if (index == _playlistIndex)
                {
                    closeFile();
                    _playlistIndex = -1;
                }
                else if (index < _playlistIndex)
                {
                    _playlistIndex--;
                }

            }
            PlayListBox.Invalidate();
        }

        private void PlayListBox_MouseUp(object sender, MouseEventArgs e)
        {
            if (e.Button == MouseButtons.Left)
            {
                int n = PlayListBox.IndexFromPoint(e.X, e.Y);
                if (n == ListBox.NoMatches)
                {
                    PlayListBox.SelectedItems.Clear();
                }
            }
            if (e.Button == MouseButtons.Right)
            {
                int n = PlayListBox.IndexFromPoint(e.X, e.Y);
                if (n != ListBox.NoMatches && !PlayListBox.GetSelected(n))
                {
                    PlayListBox.SelectedItems.Clear();
                    PlayListBox.SelectedIndex = n;
                }
            }
        }

        private void PlayListBox_DragEnter(object sender, DragEventArgs e)
        {
            if (e.Data.GetDataPresent(DataFormats.FileDrop))
            {
                e.Effect = DragDropEffects.Move;
            }
            else
            {
                e.Effect = DragDropEffects.None;
            }
        }

        private void PlayListBox_DragDrop(object sender, DragEventArgs e)
        {
            if (e.Data.GetDataPresent(DataFormats.FileDrop))
            {
                string[] files = (string[])e.Data.GetData(DataFormats.FileDrop);
                PlayListBox.Items.AddRange(files);
            }
        }

        private void mnuPlaylistAdd_Click(object sender, EventArgs e)
        {
            openFilesAndAddToPlaylist(false);
        }

        private void trbSeek_Scroll(object sender, ScrollEventArgs e)
        {
            AudioVideo.CurrentPosition = (long)SeekBar.Value;
        }

        private void tsbMute_Click(object sender, EventArgs e)
        {
            SeekBar.Enabled = !SeekBar.Enabled;
        }

        private void mnuFileExit_Click(object sender, EventArgs e)
        {
            Close();
            Application.Exit();
        }

        private void mnuHelpAbout_Click(object sender, EventArgs e)
        {
            new AboutBox().ShowDialog(this);
        }

        private void button1_Click(object sender, EventArgs e)
        {
            PlayListPanel.Visible = false;
            mnuShowPlaylist.Checked = false;
        }
    }
}
